/*
 * pdwon.c : setup EINT1 for power-down and wakeup
 *   by uratan! 2008.9.9
 */
#include "common.h"
#include "LPC214x.h"

#define SHARED_EINT3  1

/*
 * register bitmaps
 */
/* EXTINT: External Interrupt Flag register */
#define EINT0		0x00000001
#define EINT1		0x00000002
#define EINT2		0x00000004
#define EINT3		0x00000008
/* INTWAKE: Interrupt Wakeup register */
#define EXTWAKE0	0x00000001
#define EXTWAKE1	0x00000002
#define EXTWAKE2	0x00000004
#define EXTWAKE3	0x00000008
#define USBWAKE		0x00000020
#define BODWAKE		0x00004000
#define RTCWAKE		0x00008000
/* EXTMODE: External Interrupt Mode register */
#define EINT0_EDGE	0x00000001
#define EINT1_EDGE	0x00000002
#define EINT2_EDGE	0x00000004
#define EINT3_EDGE	0x00000008
/* EXTPOLAR: External Interrupt Polarity register */
#define EINT0_RISING	0x00000001
#define EINT1_RISING	0x00000002
#define EINT2_RISING	0x00000004
#define EINT3_RISING	0x00000008

/* PCON: Power Control register */
#define PC_IDL         (1<<0)
#define PC_PD          (1<<1)
#define PC_BODPDM      (1<<2)
#define PC_BOGD        (1<<3)
#define PC_BORD        (1<<4)
	/* 'PC_xPDBOD (1<<2)' is lost in UM10139 rev.2 !! */

/*
 * functions
 */
void pdown_init(void);
int power_down(int en_eint);

/*
 *
 */
void pdown_init(void)
{
#if SHARED_EINT3
	INTWAKE = 0;				/* disable wake-up function */
	EXTMODE = EINT3_EDGE;		/* EINT3 edge trigger */
	EXTPOLAR = 0;				/* EINT3 is falling edge by default */
#else /* SHARED_EINT3 */
	PINSEL0 |= (3<<6);			/* set P0.3 as EINT1 */
//	PINSEL0 |= (2<<28);			/* set P0.14 as EINT1 (for test) */

	INTWAKE = 0;				/* disable wake-up function */
	EXTMODE = EINT1_EDGE;		/* EINT1 edge trigger */
	EXTPOLAR = 0;				/* EINT1 is falling edge by default */
#endif /* SHARED_EINT3 */
		/*
		 * no need to use intr-handler to wake up operation
		 */
	return;
}

/*
 * enter LPC2148 power-down mode
 */
int power_down(int en_eint)
{
	u_long imf;
	VOLATILE u_long t;
	int wakeup_by_eint;

	imf = IntDisable();				/* disable interrupts */

#if SHARED_EINT3
	/*
	 * switch port function from RxD1 to EINT3
	 */
	PINSEL0 |= (3<<18);		/* RxD1->EINT3 */

	/*
	 * once clear request here.
	 *   (means ignore wake-up request before here)
	 */
	EXTINT = EINT3;
#else /* SHARED_EINT3 */
	/*
	 * once clear request here.
	 *   (means ignore wake-up request before here)
	 */
	EXTINT = EINT1;
#endif /* SHARED_EINT3 */

	/*
	 * wait for 'USB_need_clock' stabled. (need 2msec from DSTAT_SUSP_CH)
	 * this wait is also valid to wait UART1 job finished.
	 *   the uart baudrate goes wrong by power-down. so if uart is sending
	 *   a data just now, we shall wait for it done. only single-char-
	 *   transmitting-time is enough because interrupt is disabled here.
	 */
	for(t=0; t<(LOOP_WAIT_1msec * 3); t++)
		;

	/*
	 * select wake up source
	 */
	if(en_eint) {
#if SHARED_EINT3
		INTWAKE = USBWAKE | EXTWAKE3;	/* USB or EINT3 can wake up me */
#else /* SHARED_EINT3 */
		INTWAKE = USBWAKE | EXTWAKE1;	/* USB or EINT1 can wake up me */
#endif /* SHARED_EINT3 */
	} else {
		INTWAKE = USBWAKE;				/* only USB can wake up me */
	}

	/*
	 * then try to Power Down
	 *   this system will never enter Power Down mode if:
	 *     1) wake-up request by EINT1 is already exist.
	 *     2) USB bus activity is already recoverd (USB_need_clk=1).
	 *   (in these case, 2 PLLs keep active)
	 */
	PCON = PC_PD;
		/*
		 * I am in power down mode now! ... maybe
		 *  .
		 *  .
		 *  .
		 */
	/*
	 * I have waked up just now (but clocks are slow)
	 *   EINT1-->here is about 632usec
	 *     (start-bit of top byte HEAD_RESET is lost by this osc-startup-delay)
	 */
#if SHARED_EINT3
	wakeup_by_eint = (EXTINT & EINT3) ? 1 : 0;	/* get status */
#else /* SHARED_EINT3 */
	wakeup_by_eint = (EXTINT & EINT1) ? 1 : 0;	/* get status */
#endif /* SHARED_EINT3 */
	INTWAKE = 0;				/* disable wake-up function */

	/*
	 * re-initial both PLL if needed
	 */
	if((PLLSTAT & (1<<10)) == 0) {
		cm_sys_pll_init();		/* 47usec */
	}
	if((PLL48STAT & (1<<10)) == 0) {
		cm_usb_pll_init();		/* 43usec */
	}
		/*
		 * this system runs on slower clock for about 50usec
		 *   timer0 runs slow, but may have no problem.
		 *   uart1 transmitter is not used.
		 *   uart1 receiver is ready while HEAD_RESET(0xff), this byte
		 *         has no '0' so start-bit-mis-fetch is not occured.
		 */
#if SHARED_EINT3
	/*
	 * recover port function from EINT3 to RxD1
	 */
	PINSEL0 &= ~(2<<18);		/* EINT3->RxD1 */
#endif /* SHARED_EINT3 */

	/*
	 * recover previous state
	 */
	IntRestore(imf);				/* (may) enable interrupts */

	return(wakeup_by_eint);
}
